==> fred_reporter-0.3.0-py3-none-any.whl: reporter/client.py <==
"""DB report clients."""
import json
from abc import ABC, abstractproperty
from types import ModuleType
from typing import Dict, List, Optional, Union

import grpc
from fred_api.dbreport import service_report_manager_grpc_pb2_grpc, service_report_user_grpc_pb2_grpc
from fred_api.dbreport.common_types_pb2 import JsonEncodedValue, ReportInfo
from fred_api.dbreport.service_report_manager_grpc_pb2 import ManageReportsRequest
from fred_api.dbreport.service_report_user_grpc_pb2 import RunReportRequest
from frgal import GrpcClient, GrpcDecoder
from google.protobuf.empty_pb2 import Empty

from .report import Param, Report, ReportResult


class ReportDecoder(GrpcDecoder):
    """Report decoder."""

    def __init__(self):
        """Initialize the decoder object by registering custom decoders."""
        super().__init__()
        self.set_decoder(ReportInfo, self._decode_report_info)

    def _decode_report_info(self, value: ReportInfo) -> Report:
        """Decode Reference."""
        name = value.name.value
        input_params, output_params = [], []
        for param in value.input_parameters:
            input_params.append(Param(name=param.name.value, type=param.data_type.value))
        for param in value.output_parameters:
            output_params.append(Param(name=param.name.value, type=param.data_type.value))
        return Report(name=name, input_parameters=input_params, output_parameters=output_params)


class BaseReportClient(ABC, GrpcClient):
    """Messenger gRPC grpc-messenger-client."""

    @abstractproperty
    def grpc_module(self) -> ModuleType:
        """List of grpc service modules."""

    def __init__(self, netloc: str, credentials: grpc.ChannelCredentials = None):
        """Initialize DB report GRPC backend instance.

        Args:
            netloc: Network location of a gRPC server.
            credentials: Credentials for a secure channel connection. If None, insecure channel is used.
        """
        decoder = ReportDecoder()
        super().__init__(netloc, decoder=decoder, grpc_modules=[self.grpc_module], credentials=credentials)


class ReportClient(BaseReportClient):
    """DB report client."""

    service = 'ReportUser'
    grpc_module = service_report_user_grpc_pb2_grpc

    def list_reports(self) -> List[Report]:
        """Return list of DB reports.

        Reports with ambiguous param names are ignored.
        """
        response = self.call(self.service, 'list_reports', Empty())
        if response is None:
            return []
        else:
            return [report for report in response if report.is_unambiguous]

    def get_report(self, name: str) -> Optional[Report]:
        """Return list of DB reports.

        Reports with ambiguous param names are ignored.
        """
        for report in self.list_reports():
            if report.name == name:
                return report
        else:
            return None

    def run_report(self, name: str, *args) -> ReportResult:
        """Run DB report.

        Args:
            name: DB report name.
            *args: DB report arguments.
        """
        request = RunReportRequest()
        request.report_name.value = name
        request.arguments.extend([JsonEncodedValue(value=json.dumps(arg)) for arg in args])
        response = self.call_stream(self.service, 'run_report', request)
        return ReportResult(stream=response)


class ReportManagerClient(BaseReportClient):
    """DB report manager client."""

    service = 'ReportManager'
    grpc_module = service_report_manager_grpc_pb2_grpc

    def manage_reports(self, sql: str) -> Dict[str, Union[Report, str]]:
        """Manage db reports.

        Args:
            sql: Series of SQL commands separated by semicolon.
        """
        request = ManageReportsRequest()
        request.sql = sql
        return self.call(self.service, 'manage_reports', request)

==> fred_reporter-0.3.3-py3-none-any.whl: reporter/client.py <==
"""DB report clients."""

import json
from typing import Any, Dict, List, Optional, Union, cast

from fred_api.dbreport.common_types_pb2 import JsonEncodedValue, ReportInfo
from fred_api.dbreport.service_report_manager_grpc_pb2 import ManageReportsRequest
from fred_api.dbreport.service_report_manager_grpc_pb2_grpc import ReportManagerStub
from fred_api.dbreport.service_report_user_grpc_pb2 import RunReportRequest
from fred_api.dbreport.service_report_user_grpc_pb2_grpc import ReportUserStub
from frgal import GrpcClient, GrpcDecoder
from google.protobuf.empty_pb2 import Empty

from .report import Param, Report, ReportResult


class ReportDecoder(GrpcDecoder):
    """Report decoder."""

    decode_unset_messages = False
    decode_empty_string_none = False

    def __init__(self) -> None:
        """Initialize the decoder object by registering custom decoders."""
        super().__init__()
        self.set_decoder(ReportInfo, self._decode_report_info)

    def _decode_report_info(self, value: ReportInfo) -> Report:
        """Decode Reference."""
        name = value.name.value
        input_params, output_params = [], []
        for param in value.input_parameters:
            input_params.append(Param(name=param.name.value, type=param.data_type.value))
        for param in value.output_parameters:
            output_params.append(Param(name=param.name.value, type=param.data_type.value))
        return Report(name=name, input_parameters=input_params, output_parameters=output_params)


class ReportClient(GrpcClient):
    """DB report client."""

    stub_cls = ReportUserStub
    decoder_cls = ReportDecoder

    def list_reports(self) -> List[Report]:
        """Return list of DB reports.

        Reports with ambiguous param names are ignored.
        """
        response = self.call("list_reports", Empty())
        if response is None:
            return []
        else:
            return [report for report in response if report.is_unambiguous]

    def get_report(self, name: str) -> Optional[Report]:
        """Return list of DB reports.

        Reports with ambiguous param names are ignored.
        """
        for report in self.list_reports():
            if report.name == name:
                return report
        else:
            return None

    def run_report(self, name: str, *args: Any) -> ReportResult:
        """Run DB report.

        Args:
            name: DB report name.
            *args: DB report arguments.
        """
        request = RunReportRequest()
        request.report_name.value = name
        request.arguments.extend([JsonEncodedValue(value=json.dumps(arg)) for arg in args])
        response = self.call_stream("run_report", request)
        return ReportResult(stream=response)


class ReportManagerClient(GrpcClient):
    """DB report manager client."""

    stub_cls = ReportManagerStub
    decoder_cls = ReportDecoder

    def manage_reports(self, sql: str) -> Dict[str, Union[Report, str]]:
        """Manage db reports.

        Args:
            sql: Series of SQL commands separated by semicolon.
        """
        request = ManageReportsRequest()
        request.sql = sql
        return cast(Dict[str, Union[Report, str]], self.call("manage_reports", request))
